@@ -1,7 +1,6 @@
 import os
 import sys
 import traceback
-import json
 import functools
 import time
 import multiprocessing
@@ -32,6 +31,7 @@ def init(system_kwargs, ncores=None, **kwargs):
 
 def finalize():
     # needed to make things like pytest coverage reporting work
+    pslogger.finalize()
     pool.close()
     pool.join()
 
@@ -69,13 +69,14 @@ def do_work_wrapper(func, system_kwargs, user_kwargs, psets):
         ret = []
         for pset in psets:
             raw_stats = dict()
+            system_kwargs['raw_stats'] = raw_stats
             system_ret = {'raw_stats': raw_stats}
             user_ret = {'pset': pset}
 
             try:
-                with stats.record_wallclock(name+'_wall', raw_stats):
-                    with stats.record_iowait(name+'_io', raw_stats):
-                        result = func(pset, system_kwargs, user_kwargs, raw_stats)
+                with stats.record_wallclock(name+'_wallclock', raw_stats):
+                    with stats.record_iowait(name+'_iowait', raw_stats):
+                        result = func(pset, system_kwargs, user_kwargs)
                 user_ret['result'] = result
             except Exception as e:
                 user_ret['exception'] = repr(e)
