@@ -5,6 +5,8 @@ import pytest
 import sys
 from io import StringIO
 
+import pandas as pd
+
 import paramsurvey
 import paramsurvey.stats
 from paramsurvey.examples import sleep_worker, burn_worker
@@ -43,7 +45,7 @@ def test_basics(paramsurvey_init):
     elapsed = time.time() - start
     assert elapsed > duration, 'must take at least {} time'.format(duration)
 
-    assert [r.slept == duration for r in results], 'everyone slept '+str(duration)
+    assert [r.slept == duration for r in results.itertuples()], 'everyone slept '+str(duration)
     assert len(results) == len(psets), 'one return for each pset'
     assert len(results.missing) == 0
     assert results.progress.total == len(psets)
@@ -53,7 +55,7 @@ def test_basics(paramsurvey_init):
     assert results.progress.exceptions == 0
     assert isinstance(results.verbose, int)
 
-    df_as_listdict = results.to_listdict()
+    df_as_listdict = results.to_dict('records')
     assert len(df_as_listdict) == len(results)
     assert [d['slept'] == duration for d in df_as_listdict]
 
@@ -70,7 +72,7 @@ def test_basics(paramsurvey_init):
     assert results.progress.failures == 0
     assert results.progress.exceptions == 0
 
-    assert [r.slept == duration for r in results], 'everyone slept '+str(duration)
+    assert [r.slept == duration for r in results.itertuples()], 'everyone slept '+str(duration)
     assert len(results) == len(psets), 'one return for each pset'
 
     start = time.time()
@@ -78,14 +80,40 @@ def test_basics(paramsurvey_init):
     elapsed = time.time() - start
     assert elapsed > duration*3, 'must take at least {} time'.format(duration)
 
-    assert [r.burned == duration for r in results], 'everyone burned '+str(duration)
+    assert [r.burned == duration for r in results.itertuples()], 'everyone burned '+str(duration)
     assert len(results) == len(psets), 'one return for each pset'
 
     results = paramsurvey.map(sleep_worker, psets, name='sleep_no_results', keep_results=False)
     assert len(results) == 0
 
 
-def do_test_args(pset, system_kwargs, user_kwargs, raw_stats):
+def test_results(paramsurvey_init):
+    ncores = max(4, paramsurvey.current_core_count())
+
+    duration = 0.1
+    psets = [{'duration': duration}] * ncores * 4
+
+    results = paramsurvey.map(sleep_worker, psets, name='simple')
+
+    df = results.to_df()
+    assert isinstance(df, pd.DataFrame)
+    assert len(df) == len(results)
+    with pytest.raises(ValueError):
+        results.to_dict()
+    with pytest.raises(ValueError):
+        results.to_dict(orient='dict')
+    assert [r.slept for r in results.itertuples()] == [r['slept'] for r in results.iterdicts()]
+
+    assert len(results.missing) == 0
+    for d in results.missing.iterdicts():
+        assert False, 'this should be empty'
+
+    df = results.missing.to_df()
+    assert isinstance(df, pd.DataFrame)
+    assert len(df) == len(results.missing)
+
+
+def do_test_args(pset, system_kwargs, user_kwargs):
     # this function cannot be nested inside test_args() because nested funcs can't be pickled
     assert os.getcwd() == user_kwargs['expected_cwd'], 'chdir appears to work'
     assert 'out_subdir' in system_kwargs
@@ -147,7 +175,7 @@ def test_args(capsys, paramsurvey_init):
     assert results is None
 
 
-def do_raise(pset, system_kwargs, user_kwargs, raw_stats):
+def do_raise(pset, system_kwargs, user_kwargs):
     if 'raises' in pset and pset['raises']:
         raise ValueError('foo')
     return {'foo': 'bar'}
@@ -160,8 +188,9 @@ def test_worker_exception(capsys, paramsurvey_init):
     results = paramsurvey.map(do_raise, psets, name='test_worker_exception')
     assert len(results) == 6
     assert len(results.missing) == 1
-    assert '_exception' in results.missing[0]
-    assert '_traceback' in results.missing[0]
+    m1 = next(results.missing.iterdicts())
+    assert '_exception' in m1
+    assert '_traceback' in m1
     assert results.progress.total == 7
     assert results.progress.finished == 6
     assert results.progress.failures == 1
@@ -182,14 +211,27 @@ def test_worker_exception(capsys, paramsurvey_init):
     assert 'pset:' in log, 'our pset logline seen'
 
 
-def do_nothing(pset, system_kwargs, user_kwargs, raw_stats):
+def test_out_func_raise():
+    def out_func(user_ret, system_kwargs, user_kwargs):
+        raise ValueError()
+
+    psets = [{'duration': 0.1}] * 2
+
+    name = 'out func raises'
+
+    results = paramsurvey.map(sleep_worker, psets, out_func=out_func, name=name, verbose=2)
+
+    assert results.progress.exceptions == 2
+
+
+def do_nothing(pset, system_kwargs, user_kwargs):
     return {'foo': True}
 
 
 def test_wrapper_exception(capsys, paramsurvey_init):
     psets = [{'actually_raise': False}, {'actually_raise': True}, {'actually_raise': False}, {'actually_raise': True}, {'actually_raise': False}]
 
-    results = paramsurvey.map(do_nothing, psets, raise_in_wrapper=ValueError('test_wrapper_exception'))
+    results = paramsurvey.map(do_nothing, psets, raise_in_wrapper=ValueError('test_wrapper_exception'), verbose=0)
 
     assert len(results) == 3
     assert len(results.missing) == 2
@@ -211,7 +253,7 @@ def test_wrapper_exception(capsys, paramsurvey_init):
     assert 'failures: 2' in captured.out or 'failures: 2' in captured.err
 
 
-def bad_user_function(pset, system_kwargs, user_kwargs, raw_stats):
+def bad_user_function(pset, system_kwargs, user_kwargs):
     return 3
 
 
@@ -220,7 +262,7 @@ def test_bad_user_function(paramsurvey_init):
 
     results = paramsurvey.map(bad_user_function, psets, name='bad_user_function')
 
-    print(results.df)
+    print(results.to_df())
     print(results.missing)
     print(results.stats)
 
